#  Copyright (c) yibocat 2025 All Rights Reserved
#  Python: 3.12.7
#  Date: 2025/8/24 17:49
#  Author: yibow
#  Email: yibocat@yeah.net
#  Software: AxisFuzzy

from .decorator import contract
from .base import Contract

__all__ = ['contract', 'Contract']
