#  Copyright (c) yibocat 2025 All Rights Reserved
#  Python: 3.12.7
#  Date: 2025/8/18 18:23
#  Author: yibow
#  Email: yibocat@yeah.net
#  Software: AxisFuzzy
from pathlib import Path
from typing import Union, Any
from .config_file import Config
from .manager import ConfigManager

def get_config_manager() -> 'ConfigManager': ...
def get_config() -> Config: ...
def set_config(**kwargs: Any) -> None: ...
def load_config_file(file_path: Union[str, Path]) -> None: ...
def save_config_file(file_path: Union[str, Path]) -> None: ...
def reset_config() -> None: ...